#ifndef _KGR_PATCH_BSC1012852_LOCAL_H
#define _KGR_PATCH_BSC1012852_LOCAL_H

#include <linux/netdevice.h>
#include <linux/tipc.h>

#define TIPC_MEDIA_INFO_SIZE	32
#define MAX_NODES	4096
#define WSIZE		32
#define TIPC_MEDIA_TYPE_UDP	3

/*
 * Fix CVE-2016-8632
 *  +new define
 */
/* minimum bearer MTU */
#define TIPC_MIN_BEARER_MTU    100     /* MAX_H_SIZE + INT_H_SIZE */

struct tipc_media_addr {
	u8 value[TIPC_MEDIA_INFO_SIZE];
	u8 media_id;
	u8 broadcast;
};

struct tipc_node_map {
	u32 count;
	u32 map[MAX_NODES / WSIZE];
};

struct tipc_media {
	int (*send_msg)(struct net *net, struct sk_buff *buf,
			struct tipc_bearer *b_ptr,
			struct tipc_media_addr *dest);
	int (*enable_media)(struct net *net, struct tipc_bearer *b_ptr,
			    struct nlattr *attr[]);
	void (*disable_media)(struct tipc_bearer *b_ptr);
	int (*addr2str)(struct tipc_media_addr *addr,
			char *strbuf,
			int bufsz);
	int (*addr2msg)(char *msg, struct tipc_media_addr *addr);
	int (*msg2addr)(struct tipc_bearer *b,
			struct tipc_media_addr *addr,
			char *msg);
	int (*raw2addr)(struct tipc_bearer *b,
			struct tipc_media_addr *addr,
			char *raw);
	u32 priority;
	u32 tolerance;
	u32 window;
	u32 type_id;
	u32 hwaddr_len;
	char name[TIPC_MAX_MEDIA_NAME];
};

struct tipc_bearer {
	void __rcu *media_ptr;			/* initalized by media */
	u32 mtu;				/* initalized by media */
	struct tipc_media_addr addr;		/* initalized by media */
	char name[TIPC_MAX_BEARER_NAME];
	struct tipc_media *media;
	struct tipc_media_addr bcast_addr;
	struct rcu_head rcu;
	u32 priority;
	u32 window;
	u32 tolerance;
	u32 domain;
	u32 identity;
	struct tipc_link_req *link_req;
	char net_plane;
	int node_cnt;
	struct tipc_node_map nodes;
};

struct udp_media_addr {
	__be16	proto;
	__be16	udp_port;
	union {
		struct in_addr ipv4;
		struct in6_addr ipv6;
	};
};

struct udp_bearer {
	struct tipc_bearer __rcu *bearer;
	struct socket *ubsock;
	u32 ifindex;
	struct work_struct work;
};

/*
 * Fix CVE-2016-8632
 *  +new function
 */
/* check if device MTU is sufficient for tipc headers */
static inline bool kgr_tipc_check_mtu(struct net_device *dev, unsigned int reserve)
{
	if (dev->mtu >= TIPC_MIN_BEARER_MTU + reserve)
		return false;
	netdev_warn(dev, "MTU too low for tipc bearer\n");
	return true;
}

#endif
